unit CreateDatabaseMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, SQLDMO_TLB, ComCtrls, Buttons, ExtCtrls, Menus, ActnList;

type
  TfrmCreateDatabaseMain = class(TForm)
    OpenDialog: TOpenDialog;
    Panel1: TPanel;
    Label3: TLabel;
    reServerMessages: TRichEdit;
    Panel3: TPanel;
    sbConnected: TSpeedButton;
    Label1: TLabel;
    Server: TLabel;
    btnAddUser: TButton;
    btnCreate: TButton;
    Splitter1: TSplitter;
    PopupMenu: TPopupMenu;
    pmiClear: TMenuItem;
    btnDelete: TButton;
    reSQLScript: TRichEdit;
    Label2: TLabel;
    btnLoad: TButton;
    btnExecuteSQL: TButton;
    btnImport: TButton;
    btnQry: TButton;
    ProgressBar: TProgressBar;
    lbProgress: TLabel;
    Timer: TTimer;
    ActionList: TActionList;
    actExecuteSQL: TAction;
    actAddUser: TAction;
    cbxDBName: TComboBox;
    chxNTAuthentication: TCheckBox;
    edtServerName: TEdit;
    btnGenScript: TButton;
    procedure Load(Sender: TObject);
    procedure sbConnectedClick(Sender: TObject);
    procedure SQLServerConnServerMessage(Sender: TObject; Severity,
      MessageNumber, MessageState: Integer; var Message: OleVariant);
    procedure ThreadFinished(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnCreateClick(Sender: TObject);
    procedure StartThread(Sender: TObject);
    procedure pmiClearClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnImportClick(Sender: TObject);
    procedure btnQryClick(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure btnGenScriptClick(Sender: TObject);
    procedure cbxServerNameChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure cbxDBNameChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    SQLServerConn: _SQLServer;
    procedure SetButtosEnabled;
    procedure VisualiseProgress(Value:Boolean);
    procedure ReLoadDatabases;
  published
    procedure ExecuteSQL(Sender:TObject);
    procedure AddUser(Sender:TObject);
  end;

  TExecuteSQLThread = Class(TThread)
    FError:TStringList;
    FAction:TBasicAction;
    Failure:Boolean;
    constructor Create(Suspended:boolean);
    procedure AddMessage;
    procedure Confirm;
    procedure Execute;override;
    destructor Distroy;
  end;

var
  frmCreateDatabaseMain: TfrmCreateDatabaseMain;

implementation

uses ActiveX, PasswordDlg, AddUser, ImportIntoTable, QueryResult,
  DatabaseList {$IFDEF VER140},Variants{$ENDIF};

{$R *.DFM}

procedure TExecuteSQLThread.AddMessage;
begin
  frmCreateDatabaseMain.reServerMessages.Lines.add(Trim(FError.Text));
  FError.Clear;
end;

procedure TExecuteSQLThread.Confirm;
begin
  frmCreateDatabaseMain.Timer.Enabled:=False;
  Failure:=MessageDlg('Some failure occured.'#13#10'Continue ?',mtWarning,mbOKCancel,0)<>mrOK;
  frmCreateDatabaseMain.Timer.Enabled:=True;
end;

constructor TExecuteSQLThread.Create(Suspended: boolean);
begin
  inherited;
  FError:=TstringList.Create;
end;

destructor TExecuteSQLThread.Distroy;
begin
  FError.Free;
end;

procedure TExecuteSQLThread.Execute;
Begin
  FAction.OnExecute(Self);
End;

procedure TfrmCreateDatabaseMain.Load(Sender: TObject);
begin
  if OpenDialog.Execute then
  Begin
    reSQLScript.Lines.LoadFromFile(OpenDialog.FileName);
  End;
end;

procedure TfrmCreateDatabaseMain.ThreadFinished(Sender: TObject);
Begin
  sbConnected.Enabled:=True;
  SetButtosEnabled;
  reServerMessages.Lines.Add(TExecuteSQLThread(Sender).FError.text);
  Panel1.Cursor:=crDefault;
  Panel3.Cursor:=crDefault;
  VisualiseProgress(False);
End;

procedure TfrmCreateDatabaseMain.sbConnectedClick(Sender: TObject);
begin
  if edtServerName.Text<>'' then
  try
    if not Assigned(SQLServerConn) then
    Begin
      SQLServerConn:=coSQLServer.Create;
      if (not chxNTAuthentication.checked and (frmPasswordDlg.ShowModal=mrOk))
        or chxNTAuthentication.checked then
        sbConnected.Down:=True;
      SQLServerConn.EnableBcp:=True;
      SQLServerConn.LoginSecure:=chxNTAuthentication.checked;
      if sbConnected.Down then
        with SQLServerConn,frmPasswordDlg do
          Connect(edtServerName.Text,edtUser.Text,edtPassword.Text)
    End
    else
    Begin
      sbConnected.Down:=False;
      SQLServerConn:=nil;
    End;
  {$IFDEF VER140}
    if Assigned(SQLServerConn) then
  {$ELSE}
    if not(VarIsNull(SQLServerConn) or VarIsEmpty(SQLServerConn)) then
  {$ENDIF}
    Begin
      sbConnected.Caption:='SQL ON';
      SetButtosEnabled;
    End
    else
    Begin
      sbConnected.Caption:='SQL OFF';
      SetButtosEnabled;
    End;
  except
    ShowMessage('Connection failed:'+Exception(ExceptObject).Message);
    sbConnected.Down:=False;
    sbConnected.Caption:='SQL OFF';
  end;
  ReLoadDatabases;
end;

procedure TfrmCreateDatabaseMain.SQLServerConnServerMessage(Sender: TObject; Severity,
  MessageNumber, MessageState: Integer; var Message: OleVariant);
begin
  reServerMessages.Lines.Add(VarToStr(Message));
end;

procedure TfrmCreateDatabaseMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if not sbConnected.Enabled then
  Begin
    ShowMessage('Can not exit while SQL thread is executing');
    Action:=caNone
  End
  else
    SQLServerConn:=nil;
end;

procedure TfrmCreateDatabaseMain.btnCreateClick(Sender: TObject);
var lvTemp:string;
begin
  try
    reServerMessages.Lines.Add('------------------------------------------------');
    SQLServerConn.ExecuteImmediate('CREATE DATABASE ['+cbxDBName.Text+']',SQLDMOExec_Default,0);
    SQLServerConn.Disconnect;
    SQLServerConn.ReConnect;
    reServerMessages.Lines.Add('Creating database ('+cbxDBName.Text+') Done.');
  except
    reServerMessages.Lines.Add(Exception(ExceptObject).Message);
  end;
  lvTemp:=cbxDBName.Text;
  ReLoadDataBases;
  cbxDBName.ItemIndex:=cbxDBName.Items.IndexOf(lvTemp);
  SetButtosEnabled;
end;

procedure TfrmCreateDatabaseMain.StartThread(Sender: TObject);
begin
  if (Sender<>btnAddUser) or (frmAddUser.ShowModal=mrOk) then
  try
    Panel1.Cursor:=crSQLWait;
    Panel3.Cursor:=crSQLWait;
    reServerMessages.Lines.add('------------------------------------------------');

    VisualiseProgress(True);

    SQLServerConn.Disconnect;
    SQLServerConn.ReConnect;
    With TExecuteSQLThread.Create(True) do
    Begin
      reServerMessages.Lines.add(TButton(Sender).Hint+'... please wait.');
      FAction:=TBasicAction(FindComponent(
        'act'+Copy(TComponent(Sender).Name,4,Length(TComponent(Sender).Name)-3)));
      OnTerminate:=ThreadFinished;
      FreeOnTerminate:=True;
      Resume;
    End;
    sbConnected.Enabled:=False;
    SetButtosEnabled;
  except
    sbConnected.Enabled:=True;
    SetButtosEnabled;
    Panel1.Cursor:=crDefault;
    Panel3.Cursor:=crDefault;
    VisualiseProgress(False);
  end
end;

procedure TfrmCreateDatabaseMain.SetButtosEnabled;
begin
  btnLoad.Enabled:=sbConnected.Down;
  btnExecuteSQL.Enabled:=sbConnected.Down and sbConnected.Enabled
    and (cbxDBName.ItemIndex>-1) and (cbxDBName.Text<>'');
  btnCreate.Enabled:=sbConnected.Down and (cbxDBName.Text<>'');
  btnAddUser.Enabled:=btnExecuteSQL.Enabled;
  btnDelete.Enabled:=btnExecuteSQL.Enabled;
  btnImport.Enabled:=btnExecuteSQL.Enabled;
  btnQry.Enabled:=btnExecuteSQL.Enabled;
  btnGenScript.Enabled:=btnExecuteSQL.Enabled;
  edtServerName.Enabled:=not sbConnected.Down and sbConnected.Enabled;
  cbxDBName.Enabled:=sbConnected.Down;
  reSQLScript.ReadOnly:=not btnExecuteSQL.Enabled;
  chxNTAuthentication.Enabled:=not sbConnected.Down;
end;

procedure TfrmCreateDatabaseMain.pmiClearClick(Sender: TObject);
begin
  TMemo(PopupMenu.PopupComponent).Clear
end;

procedure TfrmCreateDatabaseMain.btnDeleteClick(Sender: TObject);
begin
  if MessageDlg('Drop Database ('+cbxDBName.Text+') ?',mtConfirmation,mbOKCancel,0)=mrOK then
  try
    reServerMessages.Lines.Add('------------------------------------------------');
    SQLServerConn.ExecuteImmediate('DROP DATABASE ['+cbxDBName.Text+']',SQLDMOExec_Default,0);
    SQLServerConn.Disconnect;
    SQLServerConn.ReConnect;
    reServerMessages.Lines.Add('Drop database ('+cbxDBName.Text+') Done.');
  except
    reServerMessages.Lines.Add(Exception(ExceptObject).Message);
  end;
  ReLoadDataBases;
  SetButtosEnabled;
end;

procedure TfrmCreateDatabaseMain.btnImportClick(Sender: TObject);
const scBulkCopy ='BULK INSERT [%s].[dbo].[%s] FROM '#39'%s'#39' WITH ('+
   'CODEPAGE = '#39'ACP'#39','+
   'DATAFILETYPE = '#39'char'#39','+
   'FIRSTROW=2,'+
   'KEEPIDENTITY)';
var lvTableName:string;
    //Bulk:_BulkCopy;
begin
  if OpenDialog.Execute then
  Begin
    lvTableName:=ExtractFileName(OpenDialog.FileName);
    lvTableName:=Copy(lvTableName,1,Length(lvTableName)-Length(ExtractFileExt(lvTableName)));
    frmImportIntoTable.TableName.Text:=lvTableName;
    if (frmImportIntoTable.ShowModal=mrOK) then
    try
      reServerMessages.Lines.Add('------------------------------------------------');
      {//SQL Server 2000 Error
      Bulk:=CoBulkCopy2.Create;
      with Bulk do
      Begin
        DataFileType:=SQLDMODataFile_TabDelimitedChar;
        DataFilePath:=OpenDialog.FileName;
        Bulk.SetCodePage(SQLDMOBCP_RAW,SQLServerConn.CodePage);
        UseExistingConnection:=True;
        IncludeIdentityValues:=True;
      End;
      SQLServerConn.Databases.Item(cbxDBName.Text,'dbo').Tables.Item(frmImportIntoTable.TableName.Text,'dbo').
      ImportData(Bulk);
      }
      SQLServerConn.ExecuteImmediate(Format(scBulkCopy,[cbxDBName.Text,frmImportIntoTable.TableName.Text,OpenDialog.FileName]),SQLDMOExec_Default,0);
      reServerMessages.Lines.Add('Import data Done.');
    except
      reServerMessages.Lines.Add(Exception(ExceptObject).Message);
    end;
  End;
end;

procedure TfrmCreateDatabaseMain.AddUser(Sender:TObject);
var log:_Login;
    usr:_User;
    sqlInsert:String;
    QRes:QueryResults;
begin
  CoInitialize(nil);
  with TExecuteSQLThread(Sender) do
  Begin
    Failure:=False;
    try
      log:=SQLServerConn.Logins.Item(frmAddUser.edtSQLUserName.Text);
    except
      try
        FError.Add('Adding SQL Login ('+frmAddUser.edtSQLUserName.Text+') to server ...');
        Synchronize(AddMessage);
        log:=CoLogin.Create;
        log.Name:=frmAddUser.edtSQLUserName.Text;
        log.Database:='master';
        log.SetPassword('',frmAddUser.edtPassword.Text);
        log.DenyNTLogin:=false;
        if frmAddUser.chxNTAuthent.Checked then
          log.Set_Type_(SQLDMOLogin_NTUser)
        else
          log.Set_Type_(SQLDMOLogin_Standard);
        SQLServerConn.Logins.Add(log);
      except
        FError.Add(Exception(ExceptObject).Message);
        Synchronize(AddMessage);
        Failure:=True;
      end
    end;
    log.Database:='master';
    try
      usr:=SQLServerConn.Databases.Item(cbxDBName.Text,'dbo').
        Users.Item(frmAddUser.edtSQLUserName.Text);
    except
      try
        FError.Add('Adding user ('+frmAddUser.edtSQLUserName.Text+') to Database('+cbxDBName.Text+') ...');
        Synchronize(AddMessage);
        SQLServerConn.Disconnect;
        SQLServerConn.ReConnect;
        usr:=coUser.Create;
        usr.Login:=frmAddUser.edtSQLUserName.Text;
        SQLServerConn.Databases.Item(cbxDBName.Text,'dbo').Users.add(usr);
      except
        FError.Add(Exception(ExceptObject).Message);
        Synchronize(AddMessage);
        Failure:=True;
      end;
    end;
    try
      FError.Add('Make db_owner account ...');
      Synchronize(AddMessage);
      SQLServerConn.Disconnect;
      SQLServerConn.ReConnect;
      SQLServerConn.Databases.Item(cbxDBName.Text,'dbo').
          DatabaseRoles.Item('db_owner').AddMember(frmAddUser.edtSQLUserName.Text);
    except
      FError.Add(Exception(ExceptObject).Message);
      Synchronize(AddMessage);
      Failure:=True;
    end;
    if frmAddUser.chxSysAdmin.Checked then
    try
      FError.Add('Make sysadmin account ...');
      Synchronize(AddMessage);
      SQLServerConn.Disconnect;
      SQLServerConn.ReConnect;
      SQLServerConn.
          ServerRoles.Item('sysadmin').AddMember(frmAddUser.edtSQLUserName.Text);
    except
      FError.Add(Exception(ExceptObject).Message);
      Synchronize(AddMessage);
      Failure:=True;
    end;
    try
      with frmAddUser do
        QRes:=SQLServerConn.Databases.Item(cbxDBName.Text,'dbo').
        ExecuteWithResults('Select Count(*) from Clerk where '+
        'FirstName = '#39+edtFirstName.Text+#39' and '+
        '(MiddleName = '#39+edtMiddleName.Text+#39' or MiddleName is null) and '+
        'LastName = '#39+edtLastName.Text+#39,0);
      if QRes.GetColumnString(1,1)<>'0' then
      Begin
        Failure:=True;
        FError.Add('Clerk allready exist.');
        Synchronize(AddMessage);
      End;
    except
      FError.Add(Exception(ExceptObject).Message);
      Synchronize(AddMessage);
      Failure:=True;
    end;
    if Failure then
      Synchronize(Confirm);
    if not Failure then
    try
      FError.Add('Create WinClerk user ...');
      Synchronize(AddMessage);
      SQLServerConn.Disconnect;
      SQLServerConn.ReConnect;
      with frmAddUser do
      sqlInsert:=
            'Insert Into Clerk '+
            '(OrganizationUnitID,FirstName,LastName,SQLServerLoginName,UserGroupID) Values '+
            '('+edtOrganizationUnit.Text+','#39+edtFirstName.Text+#39','#39+edtLastName.Text+#39','#39+edtUser.Text+#39','+edtUserGroup.Text+')';
      SQLServerConn.Databases.Item(cbxDBName.Text,'dbo').
          ExecuteImmediate(sqlInsert,SQLDMOExec_Default,0);
      FError.Add('Adding user ('+frmAddUser.edtSQLUserName.Text+') Done.');
    except
      FError.Add(Exception(ExceptObject).Message);
    end;
    log:=nil;
    usr:=nil;
  End;
  CoUnInitialize;
end;

procedure TfrmCreateDatabaseMain.ExecuteSQL(Sender:TObject);
begin
  with frmCreateDatabaseMain,TExecuteSQLThread(Sender) do
  Begin
    try
      FError.add('Done.');
      SQLServerConn.Databases.Item(cbxDBName.Text,'dbo').ExecuteImmediate(reSQLScript.Lines.Text,SQLDMOExec_ContinueOnError,0);
    except
      FError.add(Exception(ExceptObject).Message);
    end;
  End;
end;

procedure TfrmCreateDatabaseMain.btnQryClick(Sender: TObject);
var QRes:QueryResults;
    cv2,cv,lvMaxLen:integer;
    function MaxInt(A,B:Integer):integer;
    Begin
      if A>B then
        result:=A
      else
        result:=B;
    End;
begin
  with frmQueryResult.grdResult do
  try
    for cv:=0 to ColCount-1 do
      frmQueryResult.grdResult.Cols[cv].Clear;
    ColCount:=1;
    RowCount:=2;
    QRes:=SQLServerConn.Databases.Item(cbxDBName.Text,'dbo').
      ExecuteWithResults(reSQLScript.Text,0);
    if QRes.Rows>0 then
    Begin
      ColCount:=QRes.Columns;
      RowCount:=QRes.Rows+1;
      for cv:=1 to QRes.Columns do
      Begin
        Cells[cv-1,0]:=QRes.ColumnName[cv];
        lvMaxLen:=Canvas.TextWidth(Cells[cv-1,0]);
        for cv2:=1 to QRes.Rows do
        Begin
          Cells[cv-1,cv2]:=QRes.GetColumnString(cv2,cv);
          lvMaxLen:=MaxInt(lvMaxLen,Canvas.TextWidth(Cells[cv-1,cv2]));
        End;
        ColWidths[cv-1]:=lvMaxLen+4;
      End;
    End;
  except

  end;
  frmQueryResult.Show;
end;

procedure TfrmCreateDatabaseMain.TimerTimer(Sender: TObject);
begin
  ProgressBar.StepIt;
end;

procedure TfrmCreateDatabaseMain.VisualiseProgress(Value: Boolean);
begin
  Timer.Enabled:=Value;
  ProgressBar.Position:=0;
  ProgressBar.Visible:=Value;
  lbProgress.Visible:=Value;
end;

procedure TfrmCreateDatabaseMain.btnGenScriptClick(Sender: TObject);
var cv:integer;
    lvScType:TOleEnum;
    lvScript:String;
    QRes:QueryResults;
    lvDinDocTables:TStringList;
begin
  frmDatabaseList.Caption:='Stored Procedures List';
  if frmDatabaseList.ShowModal=mrOK then
  with SQLServerConn.Databases.Item(cbxDBName.Text,'dbo'),frmDatabaseList do
  Begin
    if chxFilterDinDocTypes.checked then
    Begin
      QRes:=SQLServerConn.Databases.Item(cbxDBName.Text,'dbo').
        ExecuteWithResults('select DocumentType from DocumentType',0);
      lvDinDocTables:=TStringList.Create;
      for cv:=1 to QRes.Rows do
        lvDinDocTables.Add(QRes.GetColumnString(cv,1));
      QRes:=nil;
    End;

    lvScType:=SQLDMOScript_Default or SQLDMOScript_OwnerQualify or SQLDMOScript_NoDRI;

    if chxTriggers.Checked then
      lvScType:=lvScType or SQLDMOScript_Triggers;


    if chxGenerateDrop.Checked then
      lvScType:=lvScType or SQLDMOScript_Drops;
    reSQLScript.Text:='';

    with lbTables do
    for cv:=0 to Items.Count-1 do
    if Selected[cv] and
      (not chxFilterUserTables.checked or
        (chxFilterUserTables.checked and not Tables.Item(cv+1,'dbo').SystemObject)) and
      (not chxFilterDinDocTypes.Checked or
        (lvDinDocTables.IndexOf(Items.Strings[cv])<0))
       then
    Begin
      lvScript:=lvScript+Tables.Item(cv+1,'dbo').
        Script(lvScType,null,null,SQLDMOScript2_AnsiFile);
    End;

    lvScType:=SQLDMOScript_DRI_Defaults or SQLDMOScript_DRI_PrimaryKey or SQLDMOScript_DRIWithNoCheck or SQLDMOScript_OwnerQualify;

    with lbTables do
    for cv:=0 to Items.Count-1 do
    if Selected[cv] and
      (not chxFilterUserTables.checked or
        (chxFilterUserTables.checked and not Tables.Item(cv+1,'dbo').SystemObject)) and
      (not chxFilterDinDocTypes.Checked or
        (lvDinDocTables.IndexOf(Items.Strings[cv])<0))
       then
    Begin
      lvScript:=lvScript+Tables.Item(cv+1,'dbo').
        Script(lvScType,null,null,SQLDMOScript2_AnsiFile);
    End;

    lvScType:=SQLDMOScript_DRI_ForeignKeys or SQLDMOScript_OwnerQualify;

    with lbTables do
    for cv:=0 to Items.Count-1 do
    if Selected[cv] and
      (not chxFilterUserTables.checked or
        (chxFilterUserTables.checked and not Tables.Item(cv+1,'dbo').SystemObject)) and
      (not chxFilterDinDocTypes.Checked or
        (lvDinDocTables.IndexOf(Items.Strings[cv])<0))
       then
    Begin
      lvScript:=lvScript+Tables.Item(cv+1,'dbo').
        Script(lvScType,null,null,SQLDMOScript2_AnsiFile);
    End;

    lvScType:=SQLDMOScript_Default or SQLDMOScript_OwnerQualify;
    if chxGenerateDrop.Checked then
      lvScType:=lvScType or SQLDMOScript_Drops;

    with lbStoredProcs do
    for cv:=0 to Items.Count-1 do
    if Selected[cv] and not StoredProcedures.Item(cv+1,'dbo').SystemObject then
    Begin
      lvScript:=lvScript+StoredProcedures.Item(cv+1,'dbo').
        Script(lvScType,null,SQLDMOScript2_AnsiFile);
    End;

    with lbViews do
    for cv:=0 to Items.Count-1 do
    if Selected[cv] and not Views.Item(cv+1,'dbo').SystemObject then
    Begin
      lvScript:=lvScript+Views.Item(cv+1,'dbo').
        Script(lvScType,null,SQLDMOScript2_AnsiFile);
    End;
    reSQLScript.Text:=lvScript;
  End;
  FreeAndNil(lvDinDocTables);
end;

procedure TfrmCreateDatabaseMain.cbxServerNameChange(Sender: TObject);
begin
  {$IFDEF VER140}
    if not Assigned(SQLServerConn)
  {$ELSE}
    if VarIsNull(SQLServerConn) or VarIsEmpty(SQLServerConn)
  {$ENDIF}
     and (MessageDlg('Would you like connect to this server?',mtConfirmation,mbOKCancel,0)=mrOK) then
    sbConnectedClick(sbConnected);
end;

procedure TfrmCreateDatabaseMain.ReLoadDatabases;
var cv:integer;
begin
  cbxDBName.Clear;
  with SQLServerConn do
  {$IFDEF VER140}
    if Assigned(SQLServerConn) then
  {$ELSE}
    if not(VarIsNull(SQLServerConn) or VarIsEmpty(SQLServerConn)) then
  {$ENDIF}
      for cv:=1 to Databases.Count do
        cbxDBName.Items.Add(Databases.Item(cv,'dbo').Name);
end;

procedure TfrmCreateDatabaseMain.FormShow(Sender: TObject);
begin
  if ParamCount=1 then
    reSQLScript.Lines.LoadFromFile(ParamStr(1));
end;

procedure TfrmCreateDatabaseMain.cbxDBNameChange(Sender: TObject);
begin
  SetButtosEnabled;
end;

end.
